package org.fuseri.modulelanguageschool.user;

import jakarta.persistence.EntityNotFoundException;
import lombok.Getter;
import org.fuseri.modulelanguageschool.common.DomainService;
import org.fuseri.modulelanguageschool.course.Course;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class UserService extends DomainService<User> {

    @Getter
    private final UserRepository repository;

    @Autowired
    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public User find(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("User '" + id + "' not found."));
    }

    public List<Course> getEnrolled(Long id) {
        return repository.getEnrolled(id);
    }

    public List<Course> getFinished(Long id) {
        return repository.getFinished(id);
    }

}
